@@ -17,10 +17,11 @@
 #
 ################################################################################
 import os
-import types
 import string
 import shutil
 import tempfile
+from functools import partial
+from codecs import ascii_encode
 
 from ..utils import requireUnicode, chunkCopy, datePicker
 from .. import core
@@ -32,7 +33,8 @@ from . import DEFAULT_LANG
 from . import Genre
 from . import frames
 from .headers import TagHeader, ExtendedTagHeader
-from ..compat import StringTypes, BytesType, unicode, UnicodeType
+from .. import compat
+from ..compat import StringTypes, BytesType, unicode, UnicodeType, b
 
 from ..utils.log import getLogger
 log = getLogger(__name__)
@@ -43,20 +45,21 @@ class TagException(Error):
 
 
 ID3_V1_COMMENT_DESC = u"ID3v1.x Comment"
-DEFAULT_PADDING = 1024
+DEFAULT_PADDING = 256
 
 
 class Tag(core.Tag):
-    def __init__(self):
-        core.Tag.__init__(self)
+    def __init__(self, **kwargs):
         self.clear()
+        core.Tag.__init__(self, **kwargs)
 
     def clear(self):
-        ## ID3 tag header
+        """Reset all tag data."""
+        # ID3 tag header
         self.header = TagHeader()
-        ## Optional extended header in v2 tags.
+        # Optional extended header in v2 tags.
         self.extended_header = ExtendedTagHeader()
-        ## Contains the tag's frames. ID3v1 fields are read and converted
+        # Contains the tag's frames. ID3v1 fields are read and converted
         #  the the corresponding v2 frame.
         self.frame_set = frames.FrameSet()
         self._comments = CommentsAccessor(self.frame_set)
@@ -102,7 +105,7 @@ class Tag(core.Tag):
                 if tag_found:
                     self.extended_header = None
 
-            if tag_found and self.isV2():
+            if tag_found and self.isV2:
                 self.file_info.tag_size = (TagHeader.SIZE +
                                            self.header.tag_size)
             if tag_found:
@@ -114,8 +117,8 @@ class Tag(core.Tag):
 
         return tag_found
 
-    ## returns (tag_found, padding_len)
     def _loadV2Tag(self, fp):
+        """Returns (tag_found, padding_len)"""
         padding = 0
         # Look for a tag and if found load it.
         if not self.header.parse(fp):
@@ -143,14 +146,14 @@ class Tag(core.Tag):
         fp.seek(-128, 2)
         tag_data = fp.read(128)
 
-        if tag_data[0:3] != "TAG":
+        if tag_data[0:3] != b"TAG":
             return (False, 0)
 
         log.debug("Located ID3 v1 tag")
         # v1.0 is implied until a v1.1 feature is recognized.
         self.version = ID3_V1_0
 
-        STRIP_CHARS = string.whitespace + "\x00"
+        STRIP_CHARS = compat.b(string.whitespace) + b"\x00"
         title = tag_data[3:33].strip(STRIP_CHARS)
         log.debug("Tite: %s" % title)
         if title:
@@ -178,15 +181,17 @@ class Tag(core.Tag):
             pass
 
         # Can't use STRIP_CHARS here, since the final byte is numeric
-        comment = tag_data[97:127].rstrip("\x00")
+        comment = tag_data[97:127].rstrip(b"\x00")
         # Track numbers stuffed in the comment field is what makes v1.1
         if comment:
             if (len(comment) >= 2 and
-                    comment[-2] == "\x00" and comment[-1] != "\x00"):
+                    # Python the slices (the chars), so this is really
+                    # comment[2]       and        comment[-1]
+                    comment[-2:-1] == b"\x00" and comment[-1:] != b"\x00"):
                 log.debug("Track Num found, setting version to v1.1")
                 self.version = ID3_V1_1
 
-                track = ord(comment[-1])
+                track = compat.byteOrd(comment[-1])
                 self.track_num = (track, None)
                 log.debug("Track: " + str(track))
                 comment = comment[:-2].strip(STRIP_CHARS)
@@ -215,18 +220,18 @@ class Tag(core.Tag):
         self.header.version = v
 
     def isV1(self):
-        '''Test ID3 major version for v1.x'''
+        """Test ID3 major version for v1.x"""
         return self.header.major_version == 1
 
     def isV2(self):
-        '''Test ID3 major version for v2.x'''
+        """Test ID3 major version for v2.x"""
         return self.header.major_version == 2
 
     @requireUnicode(2)
     def setTextFrame(self, fid, txt):
-        if not fid.startswith("T") or fid.startswith("TX"):
-            raise ValueError("Invalid frame-id for text frame: " +
-                             unicode(fid, "ascii"))
+        fid = b(fid, ascii_encode)
+        if not fid.startswith(b"T") or fid.startswith(b"TX"):
+            raise ValueError("Invalid frame-id for text frame")
 
         if not txt and self.frame_set[fid]:
             del self.frame_set[fid]
@@ -234,9 +239,9 @@ class Tag(core.Tag):
             self.frame_set.setTextFrame(fid, txt)
 
     def getTextFrame(self, fid):
-        if not fid.startswith("T") or fid.startswith("TX"):
-            raise ValueError("Invalid frame-id for text frame: " +
-                             unicode(fid, "ascii"))
+        fid = b(fid, ascii_encode)
+        if not fid.startswith(b"T") or fid.startswith(b"TX"):
+            raise ValueError("Invalid frame-id for text frame")
         f = self.frame_set[fid]
         return f[0].text if f else None
 
@@ -330,13 +335,16 @@ class Tag(core.Tag):
         return self._comments
 
     def _getBpm(self):
+        from decimal import Decimal, ROUND_HALF_UP, InvalidOperation
+
         bpm = None
         if frames.BPM_FID in self.frame_set:
             bpm_str = self.frame_set[frames.BPM_FID][0].text or u"0"
             try:
-                # Round floats since the spec says this is an integer
-                bpm = int(round(float(bpm_str)))
-            except ValueError as ex:
+                # Round floats since the spec says this is an integer. Python3
+                # changed how 'round' works, hence the using of decimal
+                bpm = int(Decimal(bpm_str).quantize(1, ROUND_HALF_UP))
+            except (InvalidOperation, ValueError) as ex:
                 log.warning(ex)
         return bpm
 
@@ -408,18 +416,18 @@ class Tag(core.Tag):
         return self._images
 
     def _getEncodingDate(self):
-        return self._getDate("TDEN")
+        return self._getDate(b"TDEN")
 
     def _setEncodingDate(self, date):
-        self._setDate("TDEN", date)
+        self._setDate(b"TDEN", date)
     encoding_date = property(_getEncodingDate, _setEncodingDate)
 
     @property
     def best_release_date(self):
-        '''This method tries its best to return a date of some sort, amongst
+        """This method tries its best to return a date of some sort, amongst
         alll the possible date frames. The order of preference for a release
         date is 1) date of original release 2) date of this versions release
-        3) the recording date. Or None is returned.'''
+        3) the recording date. Or None is returned."""
         import warnings
         warnings.warn("Use Tag.getBestDate() instead", DeprecationWarning,
                       stacklevel=2)
@@ -428,7 +436,7 @@ class Tag(core.Tag):
                 self.recording_date)
 
     def getBestDate(self, prefer_recording_date=False):
-        '''This method returns a date of some sort, amongst all the possible
+        """This method returns a date of some sort, amongst all the possible
         date frames. The order of preference is:
 
         1) date of original release
@@ -438,71 +446,71 @@ class Tag(core.Tag):
         Unless ``prefer_recording_date`` is ``True`` in which case the order is
         3, 1, 2.
 
-        ``None`` will be returned if no dates are available.'''
+        ``None`` will be returned if no dates are available."""
         return datePicker(self, prefer_recording_date)
 
     def _getReleaseDate(self):
-        return self._getDate("TDRL") if self.version == ID3_V2_4 \
+        return self._getDate(b"TDRL") if self.version == ID3_V2_4 \
                                      else self._getV23OrignalReleaseDate()
 
     def _setReleaseDate(self, date):
-        self._setDate("TDRL" if self.version == ID3_V2_4 else "TORY", date)
+        self._setDate(b"TDRL" if self.version == ID3_V2_4 else b"TORY", date)
 
     release_date = property(_getReleaseDate, _setReleaseDate)
-    '''The date the audio was released. This is NOT the original date the
+    """The date the audio was released. This is NOT the original date the
     work was released, instead it is more like the pressing or version of the
     release. Original release date is usually what is intended but many programs
-    use this frame and/or don't distinguish between the two.'''
+    use this frame and/or don't distinguish between the two."""
 
     def _getOrigReleaseDate(self):
-        return self._getDate("TDOR") or self._getV23OrignalReleaseDate()
+        return self._getDate(b"TDOR") or self._getV23OrignalReleaseDate()
 
     def _setOrigReleaseDate(self, date):
-        self._setDate("TDOR", date)
+        self._setDate(b"TDOR", date)
 
     original_release_date = property(_getOrigReleaseDate, _setOrigReleaseDate)
-    '''The date the work was originally released.'''
+    """The date the work was originally released."""
 
     def _getRecordingDate(self):
-        return self._getDate("TDRC") or self._getV23RecordingDate()
+        return self._getDate(b"TDRC") or self._getV23RecordingDate()
 
     def _setRecordingDate(self, date):
         if date is None:
-            for fid in ("TDRC", "TYER", "TDAT", "TIME"):
+            for fid in (b"TDRC", b"TYER", b"TDAT", b"TIME"):
                 self._setDate(fid, None)
         elif self.version == ID3_V2_4:
-            self._setDate("TDRC", date)
+            self._setDate(b"TDRC", date)
         else:
-            self._setDate("TYER", unicode(date.year))
+            self._setDate(b"TYER", unicode(date.year))
             if None not in (date.month, date.day):
                 date_str = u"%s%s" % (str(date.day).rjust(2, "0"),
                                       str(date.month).rjust(2, "0"))
-                self._setDate("TDAT", date_str)
+                self._setDate(b"TDAT", date_str)
             if None not in (date.hour, date.minute):
                 date_str = u"%s%s" % (str(date.hour).rjust(2, "0"),
                                       str(date.minute).rjust(2, "0"))
-                self._setDate("TIME", date_str)
+                self._setDate(b"TIME", date_str)
 
     recording_date = property(_getRecordingDate, _setRecordingDate)
-    '''The date of the recording. Many applications use this for release date
+    """The date of the recording. Many applications use this for release date
     regardless of the fact that this value is rarely known, and release dates
-    are more correct.'''
+    are more correct."""
 
     def _getV23RecordingDate(self):
         # v2.3 TYER (yyyy), TDAT (DDMM), TIME (HHmm)
         date = None
         try:
-            date_str = ""
-            if "TYER" in self.frame_set:
-                date_str = self.frame_set["TYER"][0].text.encode("latin1")
+            date_str = b""
+            if b"TYER" in self.frame_set:
+                date_str = self.frame_set[b"TYER"][0].text.encode("latin1")
                 date = core.Date.parse(date_str)
-            if "TDAT" in self.frame_set:
-                text = self.frame_set["TDAT"][0].text.encode("latin1")
-                date_str += "-%s-%s" % (text[2:], text[:2])
+            if b"TDAT" in self.frame_set:
+                text = self.frame_set[b"TDAT"][0].text.encode("latin1")
+                date_str += b"-%s-%s" % (text[2:], text[:2])
                 date = core.Date.parse(date_str)
-            if "TIME" in self.frame_set:
-                text = self.frame_set["TIME"][0].text.encode("latin1")
-                date_str += "T%s:%s" % (text[:2], text[2:])
+            if b"TIME" in self.frame_set:
+                text = self.frame_set[b"TIME"][0].text.encode("latin1")
+                date_str += b"T%s:%s" % (text[:2], text[2:])
                 date = core.Date.parse(date_str)
         except ValueError as ex:
             log.warning("Invalid v2.3 TYER, TDAT, or TIME frame: %s" % ex)
@@ -512,7 +520,7 @@ class Tag(core.Tag):
     def _getV23OrignalReleaseDate(self):
         date, date_str = None, None
         try:
-            for fid in ("XDOR", "TORY"):
+            for fid in (b"XDOR", b"TORY"):
                 # Prefering XDOR over TORY since it can contain full date.
                 if fid in self.frame_set:
                     date_str = self.frame_set[fid][0].text.encode("latin1")
@@ -525,10 +533,10 @@ class Tag(core.Tag):
         return date
 
     def _getTaggingDate(self):
-        return self._getDate("TDTG")
+        return self._getDate(b"TDTG")
 
     def _setTaggingDate(self, date):
-        self._setDate("TDTG", date)
+        self._setDate(b"TDTG", date)
     tagging_date = property(_getTaggingDate, _setTaggingDate)
 
     def _setDate(self, fid, date):
@@ -543,7 +551,7 @@ class Tag(core.Tag):
             return
 
         # Special casing the conversion to DATE objects cuz TDAT and TIME won't
-        if fid not in ("TDAT", "TIME"):
+        if fid not in (b"TDAT", b"TIME"):
             # Convert to ISO format which is what FrameSet wants.
             date_type = type(date)
             if date_type is int:
@@ -561,12 +569,12 @@ class Tag(core.Tag):
             self.frame_set[fid] = frames.DateFrame(fid, date_text)
 
     def _getDate(self, fid):
-        if fid in ("TORY", "XDOR"):
+        if fid in (b"TORY", b"XDOR"):
             return self._getV23OrignalReleaseDate()
 
         if fid in self.frame_set:
-            if fid in ("TYER", "TDAT", "TIME"):
-                if fid == "TYER":
+            if fid in (b"TYER", b"TDAT", b"TIME"):
+                if fid == b"TYER":
                     # Contain years only, date conversion can happen
                     return core.Date(int(self.frame_set[fid][0].text))
                 else:
@@ -600,32 +608,39 @@ class Tag(core.Tag):
     def popularities(self):
         return self._popularities
 
-    def _getGenre(self):
+    def _getGenre(self, id3_std=True):
         f = self.frame_set[frames.GENRE_FID]
         if f and f[0].text:
-            return Genre.parse(f[0].text)
+            try:
+                return Genre.parse(f[0].text, id3_std=id3_std)
+            except ValueError:
+                return None
         else:
             return None
 
-    def _setGenre(self, g):
-        '''
-        Set the genre. Four types are accepted for the ``g`` argument.
+    def _setGenre(self, g, id3_std=True):
+        """Set the genre.
+        Four types are accepted for the ``g`` argument.
         A Genre object, an acceptable (see Genre.parse) genre string,
         or an integer genre ID all will set the value. A value of None will
-        remove the genre.'''
+        remove the genre."""
         if g is None:
             if self.frame_set[frames.GENRE_FID]:
                 del self.frame_set[frames.GENRE_FID]
             return
 
         if isinstance(g, unicode):
-            g = Genre.parse(g)
+            g = Genre.parse(g, id3_std=id3_std)
         elif isinstance(g, int):
             g = Genre(id=g)
         elif not isinstance(g, Genre):
             raise TypeError("Invalid genre data type: %s" % str(type(g)))
         self.frame_set.setTextFrame(frames.GENRE_FID, unicode(g))
     genre = property(_getGenre, _setGenre)
+    """genre property."""
+    non_std_genre = property(partial(_getGenre, id3_std=False),
+                             partial(_setGenre, id3_std=False))
+    """Non-standard genres."""
 
     @property
     def user_text_frames(self):
@@ -727,12 +742,23 @@ class Tag(core.Tag):
             return self.frame_set[frames.TOS_FID][0].text
 
     @terms_of_use.setter
-    @requireUnicode(1)
     def terms_of_use(self, tos):
+        """Set the terms of use text.
+        To specify a language (other than DEFAULT_LANG) code with the text pass
+        a tuple:
+            (text, lang)
+        Language codes are 3 *bytes* of ascii data.
+        """
+        if isinstance(tos, tuple):
+            tos, lang = tos
+        else:
+            lang = DEFAULT_LANG
         if self.frame_set[frames.TOS_FID]:
             self.frame_set[frames.TOS_FID][0].text = tos
+            self.frame_set[frames.TOS_FID][0].lang = lang
         else:
-            self.frame_set[frames.TOS_FID] = frames.TermsOfUseFrame(text=tos)
+            self.frame_set[frames.TOS_FID] = frames.TermsOfUseFrame(text=tos,
+                                                                    lang=lang)
 
     def _raiseIfReadonly(self):
         if self.read_only:
@@ -740,14 +766,14 @@ class Tag(core.Tag):
 
     def save(self, filename=None, version=None, encoding=None, backup=False,
              preserve_file_time=False, max_padding=None):
-        '''Save the tag. If ``filename`` is not give the value from the
+        """Save the tag. If ``filename`` is not give the value from the
         ``file_info`` member is used, or a ``TagException`` is raised. The
         ``version`` argument can be used to select an ID3 version other than
         the version read. ``Select text encoding with ``encoding`` or use
         the existing (or default) encoding. If ``backup`` is True the orignal
         file is preserved; likewise if ``preserve_file_time`` is True the
         file´s modification/access times are not updated.
-        '''
+        """
         self._raiseIfReadonly()
 
         if not (filename or self.file_info):
@@ -787,8 +813,8 @@ class Tag(core.Tag):
         assert(version[0] == 1)
 
         def pack(s, n):
-            assert(type(s) is str)
-            return s.ljust(n, '\x00')[:n]
+            assert(type(s) is BytesType)
+            return s.ljust(n, b'\x00')[:n]
 
         def encode(s):
             return s.encode("latin_1", "replace")
@@ -800,8 +826,9 @@ class Tag(core.Tag):
         tag += pack(encode(self.album) if self.album else b"", 30)
 
         release_date = self.getBestDate()
-        year = str(release_date.year) if release_date else b""
-        tag += pack(encode(year), 4)
+        year = unicode(release_date.year).encode("ascii") if release_date \
+                                                          else b""
+        tag += pack(year, 4)
 
         cmt = ""
         for c in self.comments:
@@ -809,7 +836,7 @@ class Tag(core.Tag):
                 cmt = c.text
                 # We prefer this one over ""
                 break
-            elif c.description == "":
+            elif c.description == u"":
                 cmt = c.text
                 # Keep searching in case we find the description eyeD3 uses.
         cmt = pack(encode(cmt), 30)
@@ -817,14 +844,14 @@ class Tag(core.Tag):
         if version != ID3_V1_0:
             track = self.track_num[0]
             if track is not None:
-                cmt = cmt[0:28] + "\x00" + chr(int(track) & 0xff)
+                cmt = cmt[0:28] + b"\x00" + compat.chr(int(track) & 0xff)
         tag += cmt
 
         if not self.genre or self.genre.id is None:
             genre = 12  # Other
         else:
             genre = self.genre.id
-        tag += chr(genre & 0xff)
+        tag += compat.chr(genre & 0xff)
 
         assert(len(tag) == 128)
 
@@ -833,7 +860,7 @@ class Tag(core.Tag):
             # Write the tag over top an original or append it.
             try:
                 tag_file.seek(-128, 2)
-                if tag_file.read(3) == "TAG":
+                if tag_file.read(3) == b"TAG":
                     tag_file.seek(-128, 2)
                 else:
                     tag_file.seek(0, 2)
@@ -931,13 +958,11 @@ class Tag(core.Tag):
         header_data = self.header.render(total_size - TagHeader.SIZE)
 
         # Assemble the entire tag.
-        tag_data = b"%(tag_header)s%(ext_header)s%(frames)s" % \
-                   {"tag_header": header_data,
-                    "ext_header": ext_header_data,
-                    "frames": frame_data,
-                    }
+        tag_data = (header_data +
+                    ext_header_data +
+                    frame_data)
         assert(len(tag_data) == (total_size - padding_size))
-        return (rewrite_required, tag_data, "\x00" * padding_size)
+        return (rewrite_required, tag_data, b"\x00" * padding_size)
 
     def _saveV2Tag(self, version, encoding, max_padding):
         self._raiseIfReadonly()
@@ -1007,11 +1032,11 @@ class Tag(core.Tag):
         self.file_info.tag_size = len(tag_data) + len(padding)
 
     def _convertFrames(self, std_frames, convert_list, version):
-        '''Maps frame imcompatibilies between ID3 v2.3 and v2.4.
+        """Maps frame imcompatibilies between ID3 v2.3 and v2.4.
         The items in ``std_frames`` need no conversion, but the list/frames
         may be edited if necessary (e.g. a converted frame replaces a frame
         in the list).  The items in ``convert_list`` are the frames to convert
-        and return. The ``version`` is the target ID3 version.'''
+        and return. The ``version`` is the target ID3 version."""
         from . import versionToString
         from .frames import (DATE_FIDS, DEPRECATED_DATE_FIDS,
                              DateFrame, TextFrame)
@@ -1030,64 +1055,66 @@ class Tag(core.Tag):
 
         if date_frames:
             if version == ID3_V2_4:
-                if "TORY" in date_frames or "XDOR" in date_frames:
+                if b"TORY" in date_frames or b"XDOR" in date_frames:
                     # XDOR -> TDOR (full date)
                     # TORY -> TDOR (year only)
                     date = self._getV23OrignalReleaseDate()
                     if date:
-                        converted_frames.append(DateFrame("TDOR", date))
-                    for fid in ("TORY", "XDOR"):
+                        converted_frames.append(DateFrame(b"TDOR", date))
+                    for fid in (b"TORY", b"XDOR"):
                         if fid in flist:
                             flist.remove(date_frames[fid])
                             del date_frames[fid]
 
                 # TYER, TDAT, TIME -> TDRC
-                if ("TYER" in date_frames or "TDAT" in date_frames or
-                        "TIME" in date_frames):
+                if (b"TYER" in date_frames or b"TDAT" in date_frames or
+                        b"TIME" in date_frames):
                     date = self._getV23RecordingDate()
                     if date:
-                        converted_frames.append(DateFrame("TDRC", date))
-                    for fid in ["TYER", "TDAT", "TIME"]:
+                        converted_frames.append(DateFrame(b"TDRC", date))
+                    for fid in [b"TYER", b"TDAT", b"TIME"]:
                         if fid in date_frames:
                             flist.remove(date_frames[fid])
                             del date_frames[fid]
 
             elif version == ID3_V2_3:
-                if "TDOR" in date_frames:
-                    date = date_frames["TDOR"].date
+                if b"TDOR" in date_frames:
+                    date = date_frames[b"TDOR"].date
                     if date:
-                        converted_frames.append(DateFrame("TORY",
+                        converted_frames.append(DateFrame(b"TORY",
                                                           unicode(date.year)))
-                    flist.remove(date_frames["TDOR"])
-                    del date_frames["TDOR"]
+                    flist.remove(date_frames[b"TDOR"])
+                    del date_frames[b"TDOR"]
 
-                if "TDRC" in date_frames:
-                    date = date_frames["TDRC"].date
+                if b"TDRC" in date_frames:
+                    date = date_frames[b"TDRC"].date
 
                     if date:
-                        converted_frames.append(DateFrame("TYER",
+                        converted_frames.append(DateFrame(b"TYER",
                                                           unicode(date.year)))
                         if None not in (date.month, date.day):
                             date_str = u"%s%s" %\
                                     (str(date.day).rjust(2, "0"),
                                      str(date.month).rjust(2, "0"))
-                            converted_frames.append(TextFrame("TDAT", date_str))
+                            converted_frames.append(TextFrame(b"TDAT",
+                                                              date_str))
                         if None not in (date.hour, date.minute):
                             date_str = u"%s%s" %\
                                     (str(date.hour).rjust(2, "0"),
                                      str(date.minute).rjust(2, "0"))
-                            converted_frames.append(TextFrame("TIME", date_str))
+                            converted_frames.append(TextFrame(b"TIME",
+                                                              date_str))
 
-                    flist.remove(date_frames["TDRC"])
-                    del date_frames["TDRC"]
+                    flist.remove(date_frames[b"TDRC"])
+                    del date_frames[b"TDRC"]
 
-                if "TDRL" in date_frames:
+                if b"TDRL" in date_frames:
                     # TDRL -> XDOR
-                    date = date_frames["TDRL"].date
+                    date = date_frames[b"TDRL"].date
                     if date:
-                        converted_frames.append(DateFrame("XDOR", str(date)))
-                    flist.remove(date_frames["TDRL"])
-                    del date_frames["TDRL"]
+                        converted_frames.append(DateFrame(b"XDOR", str(date)))
+                    flist.remove(date_frames[b"TDRL"])
+                    del date_frames[b"TDRL"]
 
             # All other date frames have no conversion
             for fid in date_frames:
@@ -1096,22 +1123,23 @@ class Tag(core.Tag):
                 flist.remove(date_frames[fid])
 
         # Convert sort order frames 2.3 (XSO*) <-> 2.4 (TSO*)
-        prefix = "X" if version == ID3_V2_4 else "T"
-        fids = ["%s%s" % (prefix, suffix) for suffix in ["SOA", "SOP", "SOT"]]
+        prefix = b"X" if version == ID3_V2_4 else b"T"
+        fids = [prefix + suffix
+                    for suffix in [b"SOA", b"SOP", b"SOT"]]
         soframes = [f for f in flist if f.id in fids]
 
         for frame in soframes:
-            frame.id = ("X" if prefix == "T" else "T") + frame.id[1:]
+            frame.id = (b"X" if prefix == b"T" else b"T") + frame.id[1:]
             flist.remove(frame)
             converted_frames.append(frame)
 
         # TSIZ (v2.3) are completely deprecated, remove them
         if version == ID3_V2_4:
-            flist = [f for f in flist if f.id != "TSIZ"]
+            flist = [f for f in flist if f.id != b"TSIZ"]
 
         # TSST (v2.4) --> TIT3 (2.3)
-        if version == ID3_V2_3 and "TSST" in [f.id for f in flist]:
-            tsst_frame = [f for f in flist if f.id == "TSST"][0]
+        if version == ID3_V2_3 and b"TSST" in [f.id for f in flist]:
+            tsst_frame = [f for f in flist if f.id == b"TSST"][0]
             flist.remove(tsst_frame)
             tsst_frame = frames.UserTextFrame(
                     description=u"Subtitle (converted)", text=tsst_frame.text)
@@ -1119,7 +1147,7 @@ class Tag(core.Tag):
 
         # Raise an error for frames that could not be converted.
         if len(flist) != 0:
-            unconverted = ", ".join([f.id for f in flist])
+            unconverted = u", ".join([f.id.decode("ascii") for f in flist])
             raise TagException("Unable to covert the following frames to "
                                "version %s: %s" % (versionToString(version),
                                                    unconverted))
@@ -1199,7 +1227,7 @@ class Tag(core.Tag):
 
     @property
     def artist_origin(self):
-        '''Returns a 3-tuple: (city, state, country) Any may be ``None``.'''
+        """Returns a 3-tuple: (city, state, country) Any may be ``None``."""
         if TXXX_ARTIST_ORIGIN in self.user_text_frames:
             origin = self.user_text_frames.get(TXXX_ARTIST_ORIGIN).text
             vals = origin.split('\t')
@@ -1222,8 +1250,8 @@ class Tag(core.Tag):
             self.user_text_frames.set('\t'.join(vals), TXXX_ARTIST_ORIGIN)
 
     def frameiter(self, fids=None):
-        '''A iterator for tag frames. If ``fids`` is passed it must be a list
-        of frame IDs to filter and return.'''
+        """A iterator for tag frames. If ``fids`` is passed it must be a list
+        of frame IDs to filter and return."""
         fids = fids or []
         fids = [(b(f, ascii_encode)
             if isinstance(f, UnicodeType) else f) for f in fids]
@@ -1233,12 +1261,12 @@ class Tag(core.Tag):
 
 
 class FileInfo:
-    '''
+    """
     This class is for storing information about a parsed file. It containts info
     such as the filename, original tag size, and amount of padding; all of which
     can make rewriting faster.
-    '''
-    def __init__(self, file_name):
+    """
+    def __init__(self, file_name, tagsz=0, tpadd=0):
         from .. import LOCAL_FS_ENCODING
 
         if type(file_name) is unicode:
@@ -1253,8 +1281,8 @@ class FileInfo:
                             repr(file_name))
                 self.name = file_name
 
-        self.tag_size = 0  # This includes the padding byte count.
-        self.tag_padding_size = 0
+        self.tag_size = tagsz or 0  # This includes the padding byte count.
+        self.tag_padding_size = tpadd or 0
 
         self.initStatTimes()
 
@@ -1267,7 +1295,7 @@ class FileInfo:
             self.atime, self.mtime = s.st_atime, s.st_mtime
 
     def touch(self, times):
-        '''times is a 2-tuple of (atime, mtime).'''
+        """times is a 2-tuple of (atime, mtime)."""
         os.utime(self.name, times)
         self.initStatTimes()
 
@@ -1298,7 +1326,7 @@ class AccessorBase(object):
         return None
 
     def remove(self, *args, **kwargs):
-        '''Returns the removed item or ``None`` if not found.'''
+        """Returns the removed item or ``None`` if not found."""
         fid_frames = self._fs[self._fid] or []
         for frame in fid_frames:
             if self._match_func(frame, *args, **kwargs):
@@ -1310,7 +1338,9 @@ class AccessorBase(object):
 class DltAccessor(AccessorBase):
     def __init__(self, FrameClass, fid, fs):
         def match_func(frame, description, lang=DEFAULT_LANG):
-            return frame.description == description and frame.lang == lang
+            return (frame.description == description and
+                    frame.lang == (lang if isinstance(lang, BytesType)
+                                        else lang.encode("ascii")))
 
         super(DltAccessor, self).__init__(fid, fs, match_func)
         self.FrameClass = FrameClass
@@ -1360,15 +1390,18 @@ class ImagesAccessor(AccessorBase):
 
     @requireUnicode("description")
     def set(self, type_, img_data, mime_type, description=u"", img_url=None):
-        '''Add an image of ``type_`` (a type constant from ImageFrame).
+        """Add an image of ``type_`` (a type constant from ImageFrame).
         The ``img_data`` is either bytes or ``None``. In the latter case
         ``img_url`` MUST be the URL to the image. In this case ``mime_type``
         is ignored and "-->" is used to signal this as a link and not data
-        (per the ID3 spec).'''
+        (per the ID3 spec)."""
+        img_url = b(img_url) if img_url else None
+
         if not img_data and not img_url:
             raise ValueError("img_url MUST not be none when no image data")
 
         mime_type = mime_type if img_data else frames.ImageFrame.URL_MIME_TYPE
+        mime_type = b(mime_type)
 
         images = self._fs[frames.IMAGE_FID] or []
         for img in images:
@@ -1404,7 +1437,6 @@ class ImagesAccessor(AccessorBase):
 
 class ObjectsAccessor(AccessorBase):
     def __init__(self, fs):
-
         def match_func(frame, description):
             return frame.description == description
         super(ObjectsAccessor, self).__init__(frames.OBJECT_FID, fs, match_func)
@@ -1438,7 +1470,6 @@ class ObjectsAccessor(AccessorBase):
 
 class PrivatesAccessor(AccessorBase):
     def __init__(self, fs):
-
         def match_func(frame, owner_id):
             return frame.owner_id == owner_id
         super(PrivatesAccessor, self).__init__(frames.PRIVATE_FID, fs,
@@ -1506,7 +1537,7 @@ class UniqueFileIdAccessor(AccessorBase):
                                                    fs, match_func)
 
     def set(self, data, owner_id):
-        data = str(data)
+        data, owner_id = b(data), b(owner_id)
         if len(data) > 64:
             raise TagException("UFID data must be 64 bytes or less")
 
@@ -1523,9 +1554,11 @@ class UniqueFileIdAccessor(AccessorBase):
         return uniq_id_frame
 
     def remove(self, owner_id):
+        owner_id = b(owner_id)
         return super(UniqueFileIdAccessor, self).remove(owner_id)
 
     def get(self, owner_id):
+        owner_id = b(owner_id)
         return super(UniqueFileIdAccessor, self).get(owner_id)
 
 
@@ -1616,8 +1649,8 @@ class ChaptersAccessor(AccessorBase):
         return super(ChaptersAccessor, self).get(element_id)
 
     def __getitem__(self, elem_id):
-        '''Overiding the index based __getitem__ for one indexed with chapter
-        element IDs. These are stored in the tag's table of contents frames.'''
+        """Overiding the index based __getitem__ for one indexed with chapter
+        element IDs. These are stored in the tag's table of contents frames."""
         for chapter in (self._fs[frames.CHAPTER_FID] or []):
             if chapter.element_id == elem_id:
                 return chapter
@@ -1676,8 +1709,8 @@ class TocAccessor(AccessorBase):
         return super(TocAccessor, self).get(element_id)
 
     def __getitem__(self, elem_id):
-        '''Overiding the index based __getitem__ for one indexed with table
-        of contents element IDs.'''
+        """Overiding the index based __getitem__ for one indexed with table
+        of contents element IDs."""
         for toc in (self._fs[frames.TOC_FID] or []):
             if toc.element_id == elem_id:
                 return toc
