@@ -20,41 +20,57 @@ from __future__ import print_function
 import os
 import re
 import math
+import pathlib
 import logging
 import argparse
 import warnings
-import threading
-from ..compat import unicode, StringIO, PY2
+import magic
 
+from ..compat import unicode, PY2
+from ..utils.log import getLogger
+from .. import LOCAL_ENCODING, LOCAL_FS_ENCODING
+
+log = getLogger(__name__)
 ID3_MIME_TYPE = "application/x-id3"
 ID3_MIME_TYPE_EXTENSIONS = (".id3", ".tag")
 
-import mimetypes
-_mime_types = mimetypes.MimeTypes()
-_mime_types.readfp(StringIO("%s %s" %
-                   (ID3_MIME_TYPE,
-                    " ".join((e[1:] for e in ID3_MIME_TYPE_EXTENSIONS)))))
-del mimetypes
-del StringIO
 
-from eyed3 import LOCAL_ENCODING, LOCAL_FS_ENCODING
+class MagicTypes(magic.Magic):
+    def __init__(self):
+        magic.Magic.__init__(self, mime=True, mime_encoding=False,
+                             keep_going=False)
 
-from ..utils.log import getLogger
-log = getLogger(__name__)
+    def guess_type(self, filename):
+        if os.path.splitext(filename)[1] in ID3_MIME_TYPE_EXTENSIONS:
+            return ID3_MIME_TYPE
+        return self.from_file(filename)
+
+
+_mime_types = MagicTypes()
 
 
 def guessMimetype(filename, with_encoding=False):
-    '''Return the mime-type for ``filename``. If ``with_encoding`` is True
-    the encoding is included and a 2-tuple is returned, (mine, enc).'''
+    """Return the mime-type for ``filename``. If ``with_encoding`` is True
+    the encoding is included and a 2-tuple is returned, (mine, enc)."""
 
-    mime, enc = _mime_types.guess_type(filename, strict=False)
-    return mime if not with_encoding else (mime, enc)
+    filename = str(filename) if isinstance(filename, pathlib.Path) else filename
+    mime = _mime_types.guess_type(filename)
+    if not with_encoding:
+        return mime
+    else:
+        warnings.warn("File character encoding no lopng return, value is None",
+                      UserWarning, stacklevel=2)
+        return mime, None
 
 
 def walk(handler, path, excludes=None, fs_encoding=LOCAL_FS_ENCODING):
-    '''A wrapper around os.walk which handles exclusion patterns and unicode
-    conversion.'''
-    path = unicode(path, fs_encoding) if type(path) is not unicode else path
+    """A wrapper around os.walk which handles exclusion patterns and multiple
+    path types (unicode, pathlib.Path, bytes).
+    """
+    if isinstance(path, pathlib.Path):
+        path = str(path)
+    else:
+        path = unicode(path, fs_encoding) if type(path) is not unicode else path
 
     excludes = excludes if excludes else []
     excludes_re = []
@@ -93,33 +109,26 @@ def walk(handler, path, excludes=None, fs_encoding=LOCAL_FS_ENCODING):
 
 
 class FileHandler(object):
-    '''A handler interface for :func:`eyed3.utils.walk` callbacks.'''
+    """A handler interface for :func:`eyed3.utils.walk` callbacks."""
 
     def handleFile(self, f):
-        '''Called for each file walked. The file ``f`` is the full path and
+        """Called for each file walked. The file ``f`` is the full path and
         the return value is ignored. If the walk should abort the method should
-        raise a ``StopIteration`` exception.'''
+        raise a ``StopIteration`` exception."""
         pass
 
     def handleDirectory(self, d, files):
-        '''Called for each directory ``d`` **after** ``handleFile`` has been
+        """Called for each directory ``d`` **after** ``handleFile`` has been
         called for each file in ``files``. ``StopIteration`` may be raised to
-        halt iteration.'''
+        halt iteration."""
         pass
 
     def handleDone(self):
-        '''Called when there are no more files to handle.'''
+        """Called when there are no more files to handle."""
         pass
 
 
-def requireUnicode(*args):
-    '''Function decorator to enforce unicode argument types.
-    ``None`` is a valid argument value, in all cases, regardless of not being
-    unicode.  ``*args`` Positional arguments may be numeric argument index
-    values (requireUnicode(1, 3) - requires argument 1 and 3 are unicode)
-    or keyword argument names (requireUnicode("title")) or a combination
-    thereof.
-    '''
+def _requireArgType(arg_type, *args):
     arg_indices = []
     kwarg_names = []
     for a in args:
@@ -135,19 +144,41 @@ def requireUnicode(*args):
                 if i >= len(args):
                     # The ith argument is not there, as in optional arguments
                     break
-                if args[i] is not None and not isinstance(args[i], unicode):
-                    raise TypeError("%s(argument %d) must be unicode" %
-                                    (fn.__name__, i))
+                if args[i] is not None and not isinstance(args[i], arg_type):
+                    raise TypeError("%s(argument %d) must be %s" %
+                                    (fn.__name__, i, str(arg_type)))
             for name in kwarg_names:
                 if (name in kwargs and kwargs[name] is not None and
-                        not isinstance(kwargs[name], unicode)):
-                    raise TypeError("%s(argument %s) must be unicode" %
-                                    (fn.__name__, name))
+                        not isinstance(kwargs[name], arg_type)):
+                    raise TypeError("%s(argument %s) must be %s" %
+                                    (fn.__name__, name, str(arg_type)))
             return fn(*args, **kwargs)
         return wrapped_fn
     return wrapper
 
 
+def requireUnicode(*args):
+    """Function decorator to enforce unicode argument types.
+    ``None`` is a valid argument value, in all cases, regardless of not being
+    unicode.  ``*args`` Positional arguments may be numeric argument index
+    values (requireUnicode(1, 3) - requires argument 1 and 3 are unicode)
+    or keyword argument names (requireUnicode("title")) or a combination
+    thereof.
+    """
+    return _requireArgType(unicode, *args)
+
+
+def requireBytes(*args):
+    """Function decorator to enforce unicode argument types.
+    ``None`` is a valid argument value, in all cases, regardless of not being
+    unicode.  ``*args`` Positional arguments may be numeric argument index
+    values (requireUnicode(1, 3) - requires argument 1 and 3 are unicode)
+    or keyword argument names (requireUnicode("title")) or a combination
+    thereof.
+    """
+    return _requireArgType(bytes, *args)
+
+
 def encodeUnicode(replace=True):
     warnings.warn("use compat PY2 and be more python3", DeprecationWarning,
                   stacklevel=2)
@@ -177,10 +208,11 @@ def encodeUnicode(replace=True):
         def noop(fn):
             def call(*args, **kwargs):
                 return fn(*args, **kwargs)
-            return noop
+        return noop
+
 
 def formatTime(seconds, total=None, short=False):
-    '''
+    """
     Format ``seconds`` (number of seconds) as a string representation.
     When ``short`` is False (the default) the format is:
 
@@ -196,7 +228,7 @@ def formatTime(seconds, total=None, short=False):
 
     If ``total`` is not None it will also be formatted and
     appended to the result seperated by ' / '.
-    '''
+    """
     def time_tuple(ts):
         if ts is None or ts < 0:
             ts = 0
@@ -229,7 +261,7 @@ def formatTime(seconds, total=None, short=False):
 
         if seconds < 60:
             return u'   {0:02d}s'.format(seconds)
-        for i in xrange(len(units) - 1):
+        for i in range(len(units) - 1):
             unit1, limit1 = units[i]
             unit2, limit2 = units[i + 1]
             if seconds >= limit1:
@@ -240,21 +272,21 @@ def formatTime(seconds, total=None, short=False):
 
 
 KB_BYTES = 1024
-'''Number of bytes per KB (2^10)'''
+"""Number of bytes per KB (2^10)"""
 MB_BYTES = 1048576
-'''Number of bytes per MB (2^20)'''
+"""Number of bytes per MB (2^20)"""
 GB_BYTES = 1073741824
-'''Number of bytes per GB (2^30)'''
+"""Number of bytes per GB (2^30)"""
 KB_UNIT = "KB"
-'''Kilobytes abbreviation'''
+"""Kilobytes abbreviation"""
 MB_UNIT = "MB"
-'''Megabytes abbreviation'''
+"""Megabytes abbreviation"""
 GB_UNIT = "GB"
-'''Gigabytes abbreviation'''
+"""Gigabytes abbreviation"""
 
 
 def formatSize(size, short=False):
-    '''Format ``size`` (nuber of bytes) into string format doing KB, MB, or GB
+    """Format ``size`` (nuber of bytes) into string format doing KB, MB, or GB
     conversion where necessary.
 
     When ``short`` is False (the default) the format is smallest unit of
@@ -264,7 +296,7 @@ def formatSize(size, short=False):
         256b
         64k
         1.1G
-    '''
+    """
     if not short:
         unit = "Bytes"
         if size >= GB_BYTES:
@@ -298,7 +330,7 @@ def formatSize(size, short=False):
 
 
 def formatTimeDelta(td):
-    '''Format a timedelta object ``td`` into a string. '''
+    """Format a timedelta object ``td`` into a string. """
     days = td.days
     hours = td.seconds / 3600
     mins = (td.seconds % 3600) / 60
@@ -310,7 +342,7 @@ def formatTimeDelta(td):
 
 
 def chunkCopy(src_fp, dest_fp, chunk_sz=(1024 * 512)):
-    '''Copy ``src_fp`` to ``dest_fp`` in ``chunk_sz`` byte increments.'''
+    """Copy ``src_fp`` to ``dest_fp`` in ``chunk_sz`` byte increments."""
     done = False
     while not done:
         data = src_fp.read(chunk_sz)
@@ -322,11 +354,11 @@ def chunkCopy(src_fp, dest_fp, chunk_sz=(1024 * 512)):
 
 
 class ArgumentParser(argparse.ArgumentParser):
-    '''Subclass of argparse.ArgumentParser that adds version and log level
-    options.'''
+    """Subclass of argparse.ArgumentParser that adds version and log level
+    options."""
 
     def __init__(self, *args, **kwargs):
-        from eyed3.info import VERSION_MSG
+        from eyed3 import version as VERSION
         from eyed3.utils.log import LEVELS
         from eyed3.utils.log import MAIN_LOGGER
 
@@ -337,7 +369,7 @@ class ArgumentParser(argparse.ArgumentParser):
                 value = default
             return value
         main_logger = pop_kwarg("main_logger", MAIN_LOGGER)
-        version = pop_kwarg("version", VERSION_MSG)
+        version = pop_kwarg("version", VERSION)
 
         self.log_levels = [logging.getLevelName(l).lower() for l in LEVELS]
 
@@ -348,8 +380,8 @@ class ArgumentParser(argparse.ArgumentParser):
         self.add_argument("--version", action="version", version=version,
                           help="Display version information and exit")
 
-        self.debug_arg_group = self.add_argument_group("Debugging")
-        self.debug_arg_group.add_argument(
+        debug_group = self.add_argument_group("Debugging")
+        debug_group.add_argument(
                 "-l", "--log-level", metavar="LEVEL[:LOGGER]",
                 action=LoggingAction, main_logger=main_logger,
                 help="Set a log level. This option may be specified multiple "
@@ -357,6 +389,11 @@ class ArgumentParser(argparse.ArgumentParser):
                      "applies only to that logger, otherwise the level is set "
                      "on the top-level logger. Acceptable levels are %s. " %
                      (", ".join("'%s'" % l for l in self.log_levels)))
+        debug_group.add_argument("--profile", action="store_true",
+                                 default=False, dest="debug_profile",
+                       help="Run using python profiler.")
+        debug_group.add_argument("--pdb", action="store_true", dest="debug_pdb",
+                                 help="Drop into 'pdb' when errors occur.")
 
 
 class LoggingAction(argparse._AppendAction):
@@ -365,14 +402,13 @@ class LoggingAction(argparse._AppendAction):
         super(LoggingAction, self).__init__(*args, **kwargs)
 
     def __call__(self, parser, namespace, values, option_string=None):
-
         values = values.split(':')
         level, logger = values if len(values) > 1 else (values[0],
                                                         self.main_logger)
 
         logger = logging.getLogger(logger)
         try:
-            logger.setLevel(logging._levelNames[level.upper()])
+            logger.setLevel(logging._nameToLevel[level.upper()])
         except KeyError:
             msg = "invalid level choice: %s (choose from %s)" % \
                    (level, parser.log_levels)
@@ -383,7 +419,7 @@ class LoggingAction(argparse._AppendAction):
 
 
 def datePicker(thing, prefer_recording_date=False):
-    '''This function returns a date of some sort, amongst all the possible
+    """This function returns a date of some sort, amongst all the possible
     dates (members called release_date, original_release_date,
     and recording_date of type eyed3.core.Date).
 
@@ -395,7 +431,7 @@ def datePicker(thing, prefer_recording_date=False):
     Unless ``prefer_recording_date`` is ``True`` in which case the order is
     3, 1, 2.
 
-    ``None`` will be returned if no dates are available.'''
+    ``None`` will be returned if no dates are available."""
     if not prefer_recording_date:
         return (thing.original_release_date or
                 thing.release_date or
@@ -407,10 +443,10 @@ def datePicker(thing, prefer_recording_date=False):
 
 
 def makeUniqueFileName(file_path, uniq=u''):
-    '''The ``file_path`` is the desired file name, and it is returned if the
+    """The ``file_path`` is the desired file name, and it is returned if the
     file does not exist. In the case that it already exists the path is
     adjusted to be unique. First, the ``uniq`` string is added, and then
-    a couter is used to find a unique name.'''
+    a couter is used to find a unique name."""
 
     path = os.path.dirname(file_path)
     file = os.path.basename(file_path)
